/*
 *  gcompiler.h
 *  G
 *
 *  Created by Christopher Huff on Wed Oct 16 2002.
 *  Copyright (c) 2002 Christopher Huff. All rights reserved.
 *
 */


#ifndef GCOMPILER_H
#define GCOMPILER_H

#include <string>
#include <list>
#include <stack>

#include "g.h"
#include "gtokenizer.h"
#include "gfunction.h"

namespace g {

class GCompiler;
namespace ExpressNodes {class ExpNode; class ArithmeticNode;}

//This struct holds information for the function
//during the compiling stage.
class GFunctionInfo {
  private:
    GVMFunction * function;
    string name;
    int returnType;
    std::list<SymbolTable> symbolTables;
    
    vector<ExpressNodes::ExpNode *> expressionTrees;
    
    //data and code
    std::vector<opcode>opcodes;
    
    std::vector<GSymbol> parameters;
    
/*    GDataCounter stackSize;
    GDataCounter stackSizeCtr;
    GDataCounter parameterDataSize;
    GDataCounter localDataSize;*/
    size_t stackSize;
    size_t stackDepth;
    size_t parameterDataSize;
    size_t localDataSize;
    
    
  public:
    GFunctionInfo(GMachine & mach);
    
    void AddOp(unsigned long op) {opcodes.push_back(op);}
    void SetOp(int idx, unsigned long op) {opcodes[idx] = op;}
    int Position() {return opcodes.size();}//returns position of 1 past the end of the code
    
    size_t AddScalarConstant(double s);
    size_t AddVectorConstant(double x, double y, double z);
    size_t AddFunctionConstant(GFunction * fn);
    
    size_t ReserveScalarData();
    size_t ReserveVectorData();
    size_t ReserveFunctionData();
    
    void StackPushScalar() {stackDepth += sizeof(double); if(stackDepth > stackSize) stackSize = stackDepth;}
    void StackPushVector() {stackDepth += sizeof(double)*3; if(stackDepth > stackSize) stackSize = stackDepth;}
    void StackPushFunction() {stackDepth += sizeof(GFunction *); if(stackDepth > stackSize) stackSize = stackDepth;}
    void StackPopScalar() {/*stackDepth -= sizeof(double);*/}
    void StackPopVector() {/*stackDepth -= sizeof(double)*3;*/}
    void StackPopFunction() {/*stackDepth -= sizeof(GFunction *);*/}
    
    friend class GCompiler;
};


class GCompiler {
  private:
    GMachine & machine;
    GTokenizer * tokenizer;
    
    std::stack<GFunctionInfo *> functions;
    
    
    void Require(int tok);
    bool Allow(int tok);
    
    void CompileCommand();
    void CompileDeclaration(int storage);
    void CompileFunctionDeclaration(int storage);
    void CompileFunctionCode();
    void CompileNumericDeclaration(int storage, int type);
    
    int CompileExpression(int wantedType = kNoValue);
    ExpressNodes::ArithmeticNode * CompileExpressionTree();
    bool IsOperation();
    ExpressNodes::ArithmeticNode * CompileSingleValue();
    int CompileSymbol();
    
    void FinishCompile();
    
    void OpenScope();
    void CloseScope();
    
    void Error();
    
    void AddSymbol(const string & name, int type, int storage, size_t idx);
    const GSymbol * ResolveSymbol(const std::string & name);
    
    void AddOp(ExpressNodes::ExpNode * op);
    
  public:
    GCompiler(GMachine & mach);
    ~GCompiler();
    
    void Compile(const std::string & source);
};


}//namespace g

#endif

