/*
 *  gtokenizer.h
 *  G
 *
 *  Created by Christopher Huff on Wed Oct 16 2002.
 *  Copyright (c) 2002 Christopher Huff. All rights reserved.
 *
 */

#include <string>
#include <vector>

namespace g {

enum {
    kLCurlyBracketToken,
    kRCurlyBracketToken,
    kLParenthesesToken,
    kRParenthesesToken,
    kLAngleBracketToken,
    kRAngleBracketToken,
    kLSquareBracketToken,
    kRSquareBracketToken,
    
    kEqualToken,
    kPlusToken,
    kMinusToken,
    kStarToken,
    kSlashToken,
    kPercentToken,
    kHatToken,
    kAmpersandToken,
    kPipeToken,
    
    kSemicolonToken,
    kCommaToken,
    kDotToken,
    
    kScalarToken,
    kVectorToken,
    kFunctionToken,
    
    kWhileToken,
    kForToken,
    kDoToken,
    kIfToken,
    kElseToken,
    kReturnToken,
    
    kIdentifierToken,
    kScalarValueToken,
    
    kPrintToken,
    kDumpToken,
    kLabelToken,
    
    kEndOfCodeToken
};

struct Token {
    int id;
    string name;
    Token(int tokID, std::string tokName) {id = tokID; name = tokName;}
};

class GTokenizer {
  private:
    int strIdx;
    int strSize;
    const std::string * sourceString;
    
    std::vector<Token> tokens;
    Token * currentToken;
    Token * nextToken;
    int tokIdx;
    bool done;
    
    void EatWhite();
    
  public:
    GTokenizer(const std::string & source);
    ~GTokenizer();
    
    string StringForToken(int tok);
    
    Token & CurrentToken() {return *currentToken;}
    void GetNextToken();
    Token & PeekNextToken() {return *nextToken;}
    bool Done() {return done;}
};


}//namespace g

